;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Bar Chart
; A simple implementation of a realtime bar chart.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Author: Nathaniel Blackburn
; License: MIT
; Version: 0.1.0
; Website: http://nblackburn.uk
; Support: support@nblackburn.uk
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Structure Bar
  X.d
  Y.d
  Width.d
  Height.d
  Color.d
EndStructure

Global NewList Chart.Bar()

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Clears the canvas to prevent bleeding.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Procedure ClearCanvas(Canvas)
  
  If (StartDrawing(CanvasOutput(Canvas)))
    Box(0, 0, GadgetWidth(Canvas), GadgetHeight(Canvas), RGBA(255, 255, 255, 255))
    StopDrawing()  
  EndIf
  
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Redraws the canvas.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Procedure UpdateBars(Canvas)
  
  ClearCanvas(Canvas)
  
  If StartVectorDrawing(CanvasVectorOutput(Canvas))
  
    ForEach Chart()
      
      If(Chart()\X > GadgetX(Canvas))
      
        AddPathBox(Chart()\X, Chart()\Y, Chart()\Width, Chart()\Height)
        VectorSourceColor(Chart()\Color)
        FillPath()
        
      Else
        
        DeleteElement(Chart())
        
      EndIf
      
    Next
    
    StopVectorDrawing()
  
  EndIf
  
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Adds a bar to the chart.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Procedure AddBar(Canvas, Width, Height, Color)
  
  ForEach Chart()
    Chart()\X = Chart()\X - Chart()\Width
  Next
  
  AddElement(Chart())
  
  Chart()\X      = GadgetWidth(Canvas) - Width
  Chart()\Y      = GadgetHeight(Canvas) - Height
  Chart()\Width  = Width
  Chart()\Height = Height
  Chart()\Color  = Color
  
  UpdateBars(Canvas)
  
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Adds a random bar to the chart.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Procedure AddRandomBar(Canvas)
  
  Red    = Random(250, 200)
  Green  = Random(250, 200)
  Blue   = Random(250, 200)
  Height = Random(GadgetHeight(Canvas), 0)
  
  AddBar(Canvas, 10, Height, RGBA(Red, Green, Blue, 255))
  
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Implementation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Window = OpenWindow(#PB_Any, 0, 0, 600, 200, "Chart", #PB_Window_ScreenCentered|#PB_Window_MinimizeGadget)
Canvas = CanvasGadget(#PB_Any, 0, 0, 600, 200)
Timer  = AddWindowTimer(Window, 1, 1000)

Repeat
  
  Event = WaitWindowEvent()
  Timer = EventTimer()
  
  If (Event = #PB_Event_Timer And Timer = 1)
    AddRandomBar(Canvas)
  EndIf
  
Until Event = #PB_Event_CloseWindow